<?php

namespace Nwb\MultiDownloaderClient;

class MultiDownloaderClient
{
    private string $apiUrl = 'https://multi-dl.kub.nwb.fr';
    private string $apiKey;

    public function __construct(array $options = [])
    {
        $apiKey = $options['apiKey'] ?? getenv('MULTI_DOWNLOADER_API_KEY');

        if (!$apiKey) {
            throw new \InvalidArgumentException('API key is required');
        }

        $this->apiKey = $apiKey;

        if (!empty($options['apiUrl'])) {
            $this->apiUrl = $options['apiUrl'];
        }
    }

    public function downloadAsString(array $files)
    {
        $files = array_map(function (FileRequest $file) {
            return $file->toArray();
        }, $files);

        return $this->sendRequest($files);
    }

    public function downloadTo(string $path, array $files)
    {
        $files = array_map(function (FileRequest $file) {
            return $file->toArray();
        }, $files);

        $response = $this->sendRequest($files, [
            CURLOPT_RETURNTRANSFER => false,
            CURLOPT_FILE => fopen($path, 'w'),
        ]);

        return $response;
    }

    private function sendRequest(array $files, array $options = [])
    {
        $ch = curl_init();

        $options = array_replace($options, [
            CURLOPT_URL => $this->apiUrl . '/v2/zip',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => json_encode($files),
            CURLOPT_HTTPHEADER => [
                'Content-Type: application/json',
            ],
        ]);

        curl_setopt_array($ch, $options);

        $response = curl_exec($ch);

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if ($httpCode !== 200) {
            throw new MultiDownloaderException('Error while requesting API: ' . $response);
        }

        curl_close($ch);

        return $response;
    }
}
